;+
; NAME:
; mike_1dspec
;    Version 1.1
;
; PURPOSE:
;   Combines orders of a echfspec structure into a 1d spectrum.  
;   Outputs flux and error arrays into separate files (_F.fits, _E.fits)
;
; CALLING SEQUENCE:
;  
;   mike_1dspec, mike, setup, obj_id, side
;
; INPUTS:
;   mike      - MIKE structure
;   obj_id   -  Object ID  (e.g. 0L, 1L, etc)
;   [exp_id] -  Exposure frames (e.g. [0L, 1L])
;
; RETURNS:
;
; OUTPUTS:
;   1d flux      -   (fits file; FSpec/name_ech_F.fits)
;   1d error     -   (fits file; FSpec/name_ech_E.fits)
;
; OPTIONAL KEYWORDS:
;    /SILENT   - No text output
;    LIST=     - Use an input list to combine (required for multiple
;                night obs).  Output is in 'FSpec/tmp_ech.fits'.
;    OBJ_NM=   - Name of object in slit (a = science)
;    /STD      - Run on a standard star
;    OUTNM=    - Alternative output name for FSpec file
;    ENDTRM=   - Trim order edges by this when calculating ratios
;    MINPIX1=  - Minimum 'good' pixels to calculate fitting profile
;    ORDNM=    - 0th or 1st order fitting (1 = 1st, 2 = 0th (default))
;    SNRMIN=   - Minimum S/N per pixel for consideration in fitting
;    MINPIX2=  - Minimum 'good' pixels to calculate simple 0th order fit
;
; OPTIONAL OUTPUTS:
;
; COMMENTS: ;
; EXAMPLES:
;   mike_1dspec, mike, setup, obj_id, side
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   14-Jun-2004 Written by GEP
;-
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------


pro mike_1dspec, mike, setup, obj_id, side, exp_id, SILENT=silent, LIST=list, $
                 OBJ_NM=OBJ_NM, STD=std, OUTNM=outnm, CHK=chk, ORDNM=ordnm, $
                 SNRMIN=snrmin, FLUX=flux, KEYINDX=keyindx, MAXVAR=maxvar, $
                 NOFLUX=noflux, MINPIX1=minpix1, ENDTRM=endtrm, MINPIX2=minpix2

  if N_params() LT 4 and not keyword_set(LIST) then begin 
    print, 'Syntax - ' + $
      'mike_1dspec, mike, setup, obj_id, side, [exp_id], /SILENT ' + $
      'OBJ_NM=, ORDNM=, /CHK '
    print, '     LIST= [v1.0]'
    return
  endif

  if not keyword_set(OBJ_NM) then obj_nm = 'a'
  if not keyword_set(SNRMIN) then snrmin = 2.
  if not keyword_set(MINPIX1) then minpix1 = 300L
  if not keyword_set(MINPIX2) then minpix2 = 50L
  if not keyword_set(MAXVAR) then maxvar = 0.4

  if keyword_set (LIST) then $ 
    print, 'mike_1dspec:  2d --> 1d-ing the 1st file in the following list -- ', list
  
; Set exp
  if not keyword_set( LIST) then begin
    if not keyword_set( STD ) then begin
      allexp = where(mike.type EQ 'OBJ' AND mike.flg_anly NE 0 AND $
                     mike.setup EQ setup AND mike.obj_id EQ obj_id $
                     AND mike.side EQ side)
      if keyword_set(exp_id) then exp = allexp[exp_id] else exp=allexp
      nexp = n_elements(exp)
    endif else begin ;; STD
      exp = obj_id[0]
      nexp = 1
      obj_id = 99L
    endelse 
  endif else begin 
    obj_id = 99L
    readcol, list, files, FORMAT='A'
    nexp = n_elements(files)
  endelse

;;;;;;;;;;;;;
;; Open file

  if not keyword_set( SILENT ) then $
    print, 'mike_1dspec: Loading up the files....'
    
  if not keyword_set( LIST ) then begin
    if keyword_set ( OUTNM ) then subfil = outnm+obj_nm $
    else subfil = strtrim(mike[exp[0]].Obj,2) + obj_nm
    if side EQ 1 then clrc = '_b' else clrc = '_r'
    infil = 'FSpec/'+subfil+clrc+'.fits'
    print, 'mike_1dspec: Reading ', infil
    spec = xmrdfits(infil,1)
    head = xheadfits(infil)
  endif else begin
    spec = xmrdfits(list[0],1)
    head = xheadfits(list[0])
  endelse

;;;;;;;;;;;;
;;; begin 1d

    use = where(spec.phys_ordr NE 0,nuse)

    gd = where(spec.wave NE 0.)
    minwv = min(spec.wave[gd])
    maxwv = max(spec.wave[gd])
 
    if not keyword_set(ORDNM) then ordnm = 2

    velpix = (side EQ 1 ? 1.50d : 2.10d) * double(mike[exp[0]].colbin)
    cdelt = alog10(1.0d + velpix/299792.458d)
    npix = 80000L
    tot_wave = 10^(alog10(3000.0d) + dindgen(npix)*cdelt)
    weight = fltarr(npix)
    tot_flux = fltarr(npix)
    sig = replicate(-1.,npix)

    sxaddpar, head, 'CDELT1', cdelt
    sxaddpar, head, 'CRPIX1', 1
    sxaddpar, head, 'CTYPE1', 'LINEAR'
    sxaddpar, head, 'DC-FLAG', 1
    sxaddpar, head, 'BITPIX', -32
    sxaddpar, head, 'NAXIS', 1
    sxaddpar, head, 'NAXIS1', n_elements(tot_flux)

    medrto = fltarr(nuse)
    medrto[*] = 1.
    truerto = fltarr(nuse)
    truerto[0] = 1.

    loop1zero = floor(nuse/2.)
    loop2zero = loop1zero

    print, 'mike_1dspec: ' + strtrim(string(loop1zero),2)
;    if side EQ 1 then begin
      ;; BLUE
;      for ii=0,nuse-2 do begin
      for ii=loop1zero,nuse-2 do begin
          wu1 = where(spec.wave[*,use[ii]] NE 0.)
          wu2 = where(spec.wave[*,use[ii+1]] NE 0.)
          b1 = where(spec.wave[*,use[ii]] EQ 0.)
          spec.var[b1,use[ii]] = 0.
          b2 = where(spec.wave[*,use[ii+1]] EQ 0.)
          spec.var[b2,use[ii+1]] = 0.
          max2 = max(spec.wave[*,use[ii+1]])
          min1 = min(spec.wave[wu1,use[ii]])
          trmwv1 = where(spec.wave[*,use[ii]] LE max2 AND spec.wave[*,use[ii]] GT 0.)
          trmwv2 = where(spec.wave[*,use[ii+1]] GE min1 AND spec.wave[*,use[ii+1]] GT 0.,ntrmwv2)
          trmwv3 = where(spec.wave[*,use[ii+1]] LT min1 AND spec.wave[*,use[ii+1]] GT 0.)
          trmwv4 = where(spec.wave[*,use[ii]] GT max2 AND spec.wave[*,use[ii]] GT 0.)
          wvfit = spec.wave[trmwv1,use[ii]]
          fx1fit = spec.fx[trmwv1,use[ii]]
          sig1fit = sqrt(spec.var[trmwv1,use[ii]])
          fx2fit = spec.fx[trmwv2,use[ii+1]]
          sig2fit = sqrt(spec.var[trmwv2,use[ii+1]])
          highsnr = where(sig1fit NE 0. AND sig2fit NE 0. AND $
                          fx1fit/sig1fit GE SNRMIN AND fx2fit/sig2fit GE SNRMIN,nhigh)
;          if keyword_set( CHK ) then begin 
;              stit = 'Ordrers: Black = ' + strtrim(string(spec.phys_ordr[use[ii]]),2) + $
;                      ' Red = ' + strtrim(string(spec.phys_ordr[use[ii+1]]),2) 
;              x_splot, spec.wave[wu1,use[ii]], spec.fx[wu1,use[ii]], /block, $
;                       ytwo=spec.fx[wu2,use[ii+1]], xtwo=spec.wave[wu2,use[ii+1]], TITLE=stit
;          endif
          if nhigh GT minpix1 then begin 
              rtio = median(fx1fit[highsnr],100)/median(fx2fit[highsnr],100) 
              nwvfit = wvfit[highsnr]
              fitstr = x_setfitstrct(NITER=3L, NORD=1L, HSIG=2.5, LSIG=2.5, /FLGREJ)
              fit = x_fitrej(nwvfit, rtio, FITSTR=fitstr)
              if keyword_set( CHK ) then begin
                  stit = 'Ordrers: 1 = ' + strtrim(string(spec.phys_ordr[use[ii]]),2) + $
                          ' 2 = ' + strtrim(string(spec.phys_ordr[use[ii+1]]),2)
                  x_splot, nwvfit, rtio, ytwo=fit, /block, TITLE=stit
              endif
              med0 = median(fit)
              truerto[ii+1] = med0
              if not keyword_set (SILENT) then $
                  print, 'mike_1dspec: Orders: ' + strtrim(string(use[ii]),2) + $
                         ' and: ' + strtrim(string(use[ii+1]),2) + ' median rto: ' $
                         + strtrim(string(med0),2)
              if abs(median(medrto[0:ii])-med0) GT maxvar*median(medrto[0:ii])  AND $
                          ii GT 0 then begin 
                  med0 = median(medrto[0:ii])
                  spec.fx[*,use[ii+1]] = spec.fx[*,use[ii+1]]*med0
                  spec.var[*,use[ii+1]] = spec.var[*,use[ii+1]]*med0
              endif else begin
                  if ii EQ 0 AND med0 GT (1.+maxvar) then begin
                      if ordnm EQ 1 then begin
                          all_fit = x_calcfit(spec.wave[trmwv2,use[ii+1]], FITSTR=fitstr)
                          med0 = all_fit[ntrmwv2-1]
                          spec.fx[trmwv1,use[ii]] = spec.fx[trmwv1,use[ii]] / all_fit
                          spec.var[trmwv1,use[ii]] = spec.var[trmwv1,use[ii]] / all_fit
                          spec.fx[trmwv4,use[ii]] = spec.fx[trmwv4,use[ii]] / med0
                          spec.var[trmwv4,use[ii]] = spec.var[trmwv4,use[ii]] / med0
                      endif else begin
                          spec.fx[*,use[ii]] = spec.fx[*,use[ii]] / med0
                          spec.var[*,use[ii]] = spec.var[*,use[ii]] / med0
                      endelse
                  endif else begin
                      if ordnm EQ 1 then begin
                          all_fit = x_calcfit(spec.wave[trmwv2,use[ii+1]], FITSTR=fitstr)
                          med0 = all_fit[0]
                          spec.fx[trmwv2,use[ii+1]] = spec.fx[trmwv2,use[ii+1]]*all_fit
                          spec.var[trmwv2,use[ii+1]] = spec.var[trmwv2,use[ii+1]]*all_fit
                          spec.fx[trmwv3,use[ii+1]] = spec.fx[trmwv3,use[ii+1]]*med0
                          spec.var[trmwv3,use[ii+1]] = spec.var[trmwv3,use[ii+1]]*med0
                      endif else begin
                          spec.fx[*,use[ii+1]] = spec.fx[*,use[ii+1]]*med0
                          spec.var[*,use[ii+1]] = spec.var[*,use[ii+1]]*med0
                      endelse
                  endelse
              endelse
          endif else begin 
            if nhigh GT minpix2 then begin
              rtio = fx1fit[highsnr]/fx2fit[highsnr]
              med0 = median(rtio)
              truerto[ii+1] = med0
            endif else begin 
              med0 = median(medrto[0:ii])
              truerto[ii+1] = med0
            endelse
            if ii EQ 0 AND med0 GT (1.+maxvar) then begin
                spec.fx[*,use[ii]] = spec.fx[*,use[ii]] / med0
                spec.var[*,use[ii]] = spec.var[*,use[ii]] / med0
            endif else begin 
                if abs(median(medrto[0:ii])-med0) GT maxvar then med0 = median(medrto[0:ii])
                spec.fx[*,use[ii+1]] = spec.fx[*,use[ii+1]]*med0
                spec.var[*,use[ii+1]] = spec.var[*,use[ii+1]]*med0
            endelse
            if not keyword_set (SILENT) then $
                print, 'mike_1dspec: Orders: ' + strtrim(string(use[ii]),2) + $
                       ' and: ' + strtrim(string(use[ii+1]),2) + ' median rto: ' $
                       + strtrim(string(med0),2)
          endelse
          medrto[ii+1] = med0
          if keyword_set( CHK ) then begin 
              stit = 'Ordrers: Black = ' + strtrim(string(spec.phys_ordr[use[ii]]),2) + $
                      ' Red = ' + strtrim(string(spec.phys_ordr[use[ii+1]]),2) + $
                      ' Corrected with med rtio: ' + strtrim(string(med0)) 
              x_splot, spec.wave[wu1,use[ii]], spec.fx[wu1,use[ii]], /block, $
                       ytwo=spec.fx[wu2,use[ii+1]], xtwo=spec.wave[wu2,use[ii+1]], TITLE=stit
          endif
      endfor
;    endif else begin
      ;; RED
;      for ii=nuse-1,1,-1 do begin
      for ii=loop2zero,1,-1 do begin
          wu1 = where(spec.wave[*,use[ii]] NE 0.)
          wu2 = where(spec.wave[*,use[ii-1]] NE 0.)
          b1 = where(spec.wave[*,use[ii]] EQ 0.)
          spec.var[b1,use[ii]] = 0.
          b2 = where(spec.wave[*,use[ii-1]] EQ 0.)
          spec.var[b2,use[ii-1]] = 0.
          max2 = max(spec.wave[*,use[ii]])
          min1 = min(spec.wave[wu2,use[ii-1]])
          trmwv1 = where(spec.wave[*,use[ii]] GE min1 AND spec.wave[*,use[ii]] GT 0.)
          trmwv2 = where(spec.wave[*,use[ii-1]] LE max2 AND spec.wave[*,use[ii-1]] GT 0.,ntrmwv2)
          trmwv3 = where(spec.wave[*,use[ii-1]] GT max2 AND spec.wave[*,use[ii-1]] GT 0.)
          trmwv4 = where(spec.wave[*,use[ii]] LT min1 AND spec.wave[*,use[ii]] GT 0.)
          wvfit = spec.wave[trmwv1,use[ii]]
          fx1fit = spec.fx[trmwv1,use[ii]]
          sig1fit = sqrt(spec.var[trmwv1,use[ii]])
          fx2fit = spec.fx[trmwv2,use[ii-1]]
          sig2fit = sqrt(spec.var[trmwv2,use[ii-1]])
          highsnr = where(sig1fit NE 0. AND sig2fit NE 0. AND $
                          fx1fit/sig1fit GE SNRMIN AND fx2fit/sig2fit GE SNRMIN,nhigh)
;          if keyword_set( CHK ) then begin 
;              stit = 'Ordrers: Black = ' + strtrim(string(spec.phys_ordr[use[ii]]),2) + $
;                      ' Red = ' + strtrim(string(spec.phys_ordr[use[ii-1]]),2) 
;              x_splot, spec.wave[wu1,use[ii]], spec.fx[wu1,use[ii]], /block, $
;                       ytwo=spec.fx[wu2,use[ii-1]], xtwo=spec.wave[wu2,use[ii-1]], TITLE=stit
;          endif
          if nhigh GT minpix1 then begin 
              rtio = median(fx1fit[highsnr],100)/median(fx2fit[highsnr],100) 
              nwvfit = wvfit[highsnr]
              fitstr = x_setfitstrct(NITER=3L, NORD=1L, HSIG=2.5, LSIG=2.5, /FLGREJ)
              fit = x_fitrej(nwvfit, rtio, FITSTR=fitstr)
              if keyword_set( CHK ) then begin
                  stit = 'Ordrers: 1 = ' + strtrim(string(spec.phys_ordr[use[ii]]),2) + $
                          ' 2 = ' + strtrim(string(spec.phys_ordr[use[ii-1]]),2)
                  x_splot, nwvfit, rtio, ytwo=fit, /block, TITLE=stit
              endif
              med0 = median(fit)
              truerto[ii-1] = med0
              if not keyword_set (SILENT) then $
                  print, 'mike_1dspec: Orders: ' + strtrim(string(use[ii]),2) + $
                         ' and: ' + strtrim(string(use[ii-1]),2) + ' median rto: ' $
                         + strtrim(string(med0),2)
              if abs(median(medrto[ii:nuse-1])-med0) GT maxvar*median(medrto[ii:nuse-1]) $
                           then begin
;                          AND ii LT nuse-1 then begin 
                  med0 = median(medrto[ii:nuse-1])
                  spec.fx[*,use[ii-1]] = spec.fx[*,use[ii-1]]*med0
                  spec.var[*,use[ii-1]] = spec.var[*,use[ii-1]]*med0
              endif else begin
;                  if ii EQ nuse-1 AND med0 GT (1.+maxvar) then begin
;                      if ordnm EQ 1 then begin
;                          all_fit = x_calcfit(spec.wave[trmwv2,use[ii-1]], FITSTR=fitstr)
;                          med0 = all_fit[0]
;                          spec.fx[trmwv1,use[ii]] = spec.fx[trmwv1,use[ii]] / all_fit
;                          spec.var[trmwv1,use[ii]] = spec.var[trmwv1,use[ii]] / all_fit
;                          spec.fx[trmwv4,use[ii]] = spec.fx[trmwv4,use[ii]] / med0
;                          spec.var[trmwv4,use[ii]] = spec.var[trmwv4,use[ii]] / med0
;                      endif else begin
;                          spec.fx[*,use[ii]] = spec.fx[*,use[ii]] / med0
;                          spec.var[*,use[ii]] = spec.var[*,use[ii]] / med0
;                      endelse
;                  endif else begin
                      if ordnm EQ 1 then begin
                          all_fit = x_calcfit(spec.wave[trmwv2,use[ii-1]], FITSTR=fitstr)
                          med0 = all_fit[ntrmwv2-1]
                          spec.fx[trmwv2,use[ii-1]] = spec.fx[trmwv2,use[ii-1]]*all_fit
                          spec.var[trmwv2,use[ii-1]] = spec.var[trmwv2,use[ii-1]]*all_fit
                          spec.fx[trmwv3,use[ii-1]] = spec.fx[trmwv3,use[ii-1]]*med0
                          spec.var[trmwv3,use[ii-1]] = spec.var[trmwv3,use[ii-1]]*med0
                      endif else begin
                          spec.fx[*,use[ii-1]] = spec.fx[*,use[ii-1]]*med0
                          spec.var[*,use[ii-1]] = spec.var[*,use[ii-1]]*med0
                      endelse
;                  endelse
              endelse
          endif else begin 
            if nhigh GT minpix2 then begin
              rtio = fx1fit[highsnr]/fx2fit[highsnr]
              med0 = median(rtio)
              truerto[ii-1] = med0
            endif else begin 
              med0 = median(medrto[ii:nuse-1])
              truerto[ii-1] = med0
            endelse
            if ii EQ nuse-1 AND med0 GT (1.+maxvar) then begin
                spec.fx[*,use[ii]] = spec.fx[*,use[ii]] / med0
                spec.var[*,use[ii]] = spec.var[*,use[ii]] / med0
            endif else begin 
                if abs(median(medrto[ii:nuse-1])-med0) GT maxvar then med0 = median(medrto[ii:nuse-1])
                spec.fx[*,use[ii-1]] = spec.fx[*,use[ii-1]]*med0
                spec.var[*,use[ii-1]] = spec.var[*,use[ii-1]]*med0
            endelse
            if not keyword_set (SILENT) then $
                print, 'mike_1dspec: Orders: ' + strtrim(string(use[ii]),2) + $
                       ' and: ' + strtrim(string(use[ii-1]),2) + ' median rto: ' $
                       + strtrim(string(med0),2)
          endelse
          medrto[ii-1] = med0
          if keyword_set( CHK ) then begin 
              stit = 'Ordrers: Black = ' + strtrim(string(spec.phys_ordr[use[ii]]),2) + $
                      ' Red = ' + strtrim(string(spec.phys_ordr[use[ii-1]]),2) + $
                      ' Corrected with med rtio: ' + strtrim(string(med0)) 
              x_splot, spec.wave[wu1,use[ii]], spec.fx[wu1,use[ii]], /block, $
                       ytwo=spec.fx[wu2,use[ii-1]], xtwo=spec.wave[wu2,use[ii-1]], TITLE=stit
          endif
      endfor
;    endelse
    
    if keyword_set ( CHK ) then $
      x_splot, truerto, ytwo=medrto, /block

    for ii=0,nuse-1 do begin
        indx = where(abs(tot_wave - spec.wave[0,use[ii]]) LT 0.006, nindx)
        if nindx EQ 0 then stop
        indx = indx[0]

        a = where(spec.var[*,use[ii]] GT 0, ngd)
        if ngd NE 0 then begin
            wtmp = spec.var[a,use[ii]]
            tmp = tot_flux[indx+a]+spec.fx[a,use[ii]] / wtmp
            for i=0L, ngd-1 do begin
                tot_flux[indx+a[i]] = tmp[i]
                weight[indx+a[i]] = weight[indx+a[i]] + 1./wtmp[i]
            endfor
        endif
    endfor
    a = where(weight NE 0., na)
    if na EQ 0 then stop
    tot_flux[a] = tot_flux[a] / weight[a]
    sig[a] = sqrt(1./weight[a])
    
    if keyword_set( CHK ) then $
      x_splot, tot_wave[a], tot_flux[a], ytwo=sig[a], /block
    minwv = min(tot_wave[a])


    sxaddpar, head, 'CRVAL1', alog10(minwv)

    if not keyword_set( LIST ) then begin
      if keyword_set ( OUTNM ) then subfil = outnm+obj_nm $
      else subfil = strtrim(mike[exp[0]].Obj,2) + obj_nm
      if side EQ 1 then clrc = '_b' else clrc = '_r'
      outfil = 'FSpec/'+subfil+clrc+'_F.fits'
      sigfil = 'FSpec/'+subfil+clrc+'_E.fits'
    endif else begin
      outfil = list[1]
      sigfil = list[2]
    endelse

    if not keyword_set( SILENT ) then $
      print, 'mike_1dspec: Writing results to files: ', outfil, sigfil

    mwrfits, tot_flux[a], outfil, head, /create, /silent
    mwrfits, sig[a], sigfil, head, /create, /silent

end
