;+ 
; NAME:
; mike_calibstd   
;   Version 1.1
;
; PURPOSE:
;    Create a sensitivity function given a standard star and its
;    appropriate calibration file.  
;
; CALLING SEQUENCE:
; mike_calibstd, mike, indx, HSTFIL=, CHKFIT=, ESOFIL=
;   
; INPUTS:
;   mike   -  MIKE structure
;   indx   -  Index of the Standard star in the MIKE structure to process
;
; RETURNS:
;
; OUTPUTS:
;  Sensitivity file in the 'Extract/' directory
;
; OPTIONAL KEYWORDS:
;  OUTFIL - Name of the output file (default:
;           'Extract/Sens_{mikeroot}.fits' )
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   mike_calibstd, mike, 1, ESOFIL='fhr4469.dat'
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   04-Jan-2004 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro mike_calibstd, mike, indx, HSTFIL=hstfil, CHKFIT=chkfit, $
                BSPLIN=bsplin, SWV=swv, SFX=sfx, ESOFIL=esofil, $
                BSET=bset, YFIT=yfit, SENS=sens, EVERYN=everyn, OBJFIL=objfil

;
  if  N_params() LT 2  then begin 
      print,'Syntax - ' + $
        'mike_calibstd, mike, indx, HSTFIL=, FXFIL= [v1.0]'
    return
  endif 

;  Optional Keywords
  if n_elements(indx) NE 1 then stop
  if not keyword_set( EVERYN ) then everyn = 36/mike[indx].rowbin
  if not keyword_set( OBJFIL ) then objfil = mike[indx].obj_fil
  if not keyword_set( OUTFIL ) then begin
      slen = strlen(mike[indx].img_root)
      outfil = 'Extract/Sens_'+mike[indx].img_root
  endif

; HST
  ;; Read file
  if keyword_set( HSTFIL ) then begin
      hst = xmrdfits(hstfil, 1, /silent)
      swv = hst.wavelength
      sfx = hst.flux
  endif

  if keyword_set( ESOFIL ) then begin
      if strlen(ESOFIL) LT 30 then $
        esofil = getenv('MIKE_DIR')+'/pro/Std/ESOFIL/'+esofil
      readcol, esofil, swv, sfx
      sfx = sfx*1d-16
  endif

  objstr = xmrdfits(objfil, 1, STRUCTYP='mikeobjstrct', /silent)

; Create summed array
  
  velpix = (mike[indx].side EQ 1 ? 1.50d : 2.10d) * mike[indx].rowbin
  loglam = alog10(1.0d + velpix / 299792.458d)
  wave0  = alog10(3000.0d)
  npix = 100000L
  wvtot = 10^(wave0 + dindgen(npix)*loglam)
  fxtot = dblarr(npix)
  
  ;; Loop
  for qq=0L,n_elements(objstr)-1 do begin
      gd = where(objstr[qq].var GT 0.,ngd)
      ii = where(abs(wvtot-objstr[qq].wave[gd[0]]) LT 0.01, nii)
      if nii NE 1 then stop
      ii = ii[0]
      np = gd[ngd-1]-gd[0] + 1L
      ;; Sum it in
      fxtot[ii:ii+np-1] = fxtot[ii:ii+np-1] + objstr[qq].fx[gd[0]:gd[0]+np-1]
  endfor
      

  
; Loop on orders

  for qq=0L,n_elements(objstr)-1 do begin
      ;; 
      npix = objstr[qq].npix
      wave = objstr[qq].wave[0:npix-1]
      flux = objstr[qq].fx[0:npix-1]
      sig = sqrt(objstr[qq].var[0:npix-1])

      ;; Dwv
      npix = objstr[qq].npix
      if wave[1] GT wave[0] then begin
          dwv = (shift(wave,-1)-shift(wave, 1))/2.
          dwv[0] = wave[1]-wave[0]
          dwv[npix-1] = wave[npix-1]-wave[npix-2]
      endif else begin
          dwv = (shift(wave,1)-shift(wave, -1))/2.
          dwv[0] = wave[0]-wave[1]
          dwv[npix-1] = wave[npix-2]-wave[npix-1]
      endelse

      ;; BSpline
      if keyword_set( BSPLIN ) then begin
          bset = bspline_iterfit(swv, sfx, yfit=yfit, everyn=everyn, /silent)
          if keyword_set( CHKFIT ) then begin
              x_splot, swv, sfx, ytwo=yfit, /block
              stop
          endif
          ;; Calculate at wavelength
          sens = bspline_valu(wave, bset) / (flux / mike[indx].exp / dwv)
          
      endif else begin ;; SPLINE
          ;; Calculate at wavelength
          splin = spl_init(swv, sfx, /double)
          std = spl_interp(swv, sfx, splin, wave)
          
          ;; Sens function
          inv_sens = (flux / mike[indx].exp / dwv) / std
          inv_sens_ivar = (std * mike[indx].exp * dwv)^2/(sig^2 + (sig EQ 0)) * (sig GT 0)
          objstr[qq].flux[0:npix-1] = inv_sens
          objstr[qq].sig[0:npix-1] = inv_sens_ivar
if qq EQ 5 then stop
      endelse

      continue
stop

      ;; Mask
      good = where( sig GT 0., ngood)
      if ngood EQ 0 then stop
      srt = sort(wave)
      wave = wave[srt]
      sens = sens[srt]
      inv_sens = inv_sens[srt]
      inv_sens_ivar = inv_sens_ivar[srt]
  
      ;; BSpline
      ivar = fltarr(n_elements(sens))
      ivar[*] = 1.

      ;; Zero out neg values (for bluest orders)
      a = where(sens LE 0., na)
      if na NE 0 then ivar[a] = 0.

      ;; Abs
;      bd = where((wave GT 6864. AND wave LT 6920.) OR $
;                 (wave GT 7585. AND wave LT 7684.), nbd)
;      if nbd NE 0 then ivar[bd] = -1.
      
      bset = bspline_iterfit(wave, inv_sens, yfit=yfit, everyn=everyn, /groupbadpix, maxrej=5, $
                         lower=5., upper=5., grow=21, invvar=inv_sens_ivar, /silent, outmask=outmask)

      next_mask = smooth(outmask, 61)
      bset = bspline_iterfit(wave, inv_sens, yfit=yfit2, everyn=everyn, /groupbadpix, maxrej=5, $
                             lower=2., upper=2., invvar=inv_sens_ivar*next_mask, /silent, outmask=outmask2)

      bset = bspline_iterfit(wave, sens, yfit=yfit, everyn=everyn, maxiter=1, $
                             lower=2., upper=2., invvar=ivar, /silent)
      
      svset = { $
                fullbkpt: bset.fullbkpt, $
                bkmask: bset.bkmask, $
                nord: bset.nord, $
                coeff: bset.coeff, $
                icoeff: bset.icoeff, $
                ordr: objstr[qq].order $
              }

      if keyword_set( CHKFIT ) then x_splot, wave, sens, ytwo=yfit, /block
      
      ;; Save
      if qq EQ 0 then mwrfits, svset, outfil, /create else $
        mwrfits, svset, outfil
  endfor

stop

  print, 'mike_calibstd: All Done!'
  return
end
